/*
 * @lasPlotFrame.java Version 1.1 4/25/2017
 *
 * Copyright (c) 2017 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package las.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.image.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

import gui.guiSearchDirectoryFrame;         // File Search Dialog

import iqstrat.iqstratControlStruct;        // Plot Control Data Structure
import iqstrat.iqstratHeadersStruct;        // Headers Data Structure

import plot.plotXYStruct;                   // XY Plot Data Structure
import plot.plotSymbolStruct;               // Plot Symbol Structure

import las.lasFileDataStruct;               // LAS File Data Structure
import las.lasPlotFilterListStruct;         // LAS Plot Filter List Data Structure
import las.plot.lasPlot;                    // LAS Plot

/** Class lasPlotFrame
 *  <p> This class will display the 2D Plot Frame
 *
 *  @version 1.1 4/25/2017
 *  @author  John R. Victorine
 */

public class lasPlotFrame extends JFrame implements ActionListener, Observer
{
  private static final int OFF    = cmn.cmnStruct.OFF;
  private static final int ON     = cmn.cmnStruct.ON;

  // Input Variables

  private int                     iAzimuth  = 0;
  private int                     iCalipers = 0;
  private double                  degrees   = 0.0;

  private iqstratControlStruct    stControl = null; // Plot Control Data Structure
  private iqstratHeadersStruct    stHeader  = null; // Header Data Structure
  private lasFileDataStruct       stLAS     = null; // LAS File Data Struct

  private plotXYStruct            stPlot    = null; // XY Plot Data Structure

  // Global Variables

  private Observable              notifier  = null; // Observalbles Object
  private lasPlot                 panel     = null; // LAS Borehole Plot

  private lasPlotFilterListStruct stLegend  = null; // Legend Plot Data Filter Structure

  // -- Save Path & Filename

  private String                  sPath     = new String("");
  private String                  sName     = new String("");
  private guiSearchDirectoryFrame pSearch   = null; // Search for Directory to Save PNG

  // -- raw data

  private int    iRows    = 0;
  private double data[][] = null;

  // -- Depth Ranges

  private double dStart  = 0.0;
  private double dEnd    = 0.0;

  private double dOStart = 0.0;
  private double dOEnd   = 0.0;

  // -- Reference Radius

  private double dRadius = 3.9375; // half of 7 7/8 for standard bit size

  // -- Plot Limits

  private String sTitle1 = "Kansas Geological Survey";   // First line of the plot title
  private String sTitle2 = "";   // Second line of the plot title
  private String sTitle3 = "";   // Third line of the plot title

  private String sXAxis  = "";
  private double dXmax   = 0.0;  // Maximum X-Axis
  private double dXmin   = 0.0;  // Minimum X-Axis
  private double dXincr  = 0.0;  // Increment along X-Axis

  private String sYAxis  = "";
  private double dYmax   = 0.0;  // Maximum Y-Axis
  private double dYmin   = 0.0;  // Minimum Y-Axis
  private double dYincr  = 0.0;  // Increment along Y-Axis

  // Global Frame Widgets

  // .. Plot Titles

  private JTextField txtTitle1 = new JTextField();
  private JTextField txtTitle2 = new JTextField();
  private JTextField txtTitle3 = new JTextField();

  // -- File Button Menu

  private JButton btnPDF   = null; // Save Data as PNG with option to create PDF
  private JButton btnClose = null; // Close Dialog

  // -- Depth Range

  private JTextField txtStart  = new JTextField();  // Starting Depth of Plot
  private JTextField txtEnd    = new JTextField();  // Ending Depth of Plot

  private JButton    btnReset  = new JButton();
  private JButton    btnModify = new JButton();

  // -- Reference Radius

  private JTextField txtMax    = new JTextField(); // Maximum Radius
  private JTextField txtIncr   = new JTextField(); // Radius Increment

  // -- Reference Radius

  private JTextField txtRadius = new JTextField(); // Reference Radius

  // -- Sampling of Log Data

  private int          irb   = 3;
  private String       srb[] = { "All", "5'", "10'" };
  private JRadioButton rb[]  = null;

  // -- Limits Frame

  private int iScreenWidth  = 600;
//  private JButton btnLimits = new JButton();
//  private JFrame  fLimits   = null; // Limits Frame

  /** Constructor lasPlotFrame()
   *  <p> This is the constructor for this class
   * @param stControl = Plot Control Data Structure
   * @param stHeader  = Header Information Data Structure
   * @param stLAS     = LAS File Data Structure
   */

  public lasPlotFrame( iqstratControlStruct    stControl,
                       iqstratHeadersStruct    stHeader,
                       // LAS Data
                       lasFileDataStruct       stLAS )
  {
    try
    {
	  this.stControl = stControl;
	  this.stHeader  = stHeader;
	  this.stLAS     = las.lasFileDataUtility.removeNull(stLAS);

      setTitles( this.stHeader );

      iRows    = count_XYZ( las.lasFileDataUtility.filterBy(this.stLAS, 10.0) );
      data     = convert_XYZ( iRows, las.lasFileDataUtility.filterBy(this.stLAS, 10.0) );
      stLegend = las.lasPlotFilterUtility.getFilter(data[0][2], data[iRows-1][2],
                                             las.lasPlotFilterUtility.TOTAL);
      dStart   = data[0][2];
      dOStart  = data[0][2];
      dEnd     = data[iRows-1][2];
      dOEnd    = data[iRows-1][2];

      stPlot = setPlotStruct( iRows, data, stLegend, sTitle1, sTitle2 );

      jbInit();
      if (panel != null)
      {
		panel.setDepths( dStart,  dEnd);
		panel.setRadius( dRadius );
	  }

      addWindowListener( new lasPlotFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Constructor lasPlotFrame()
   *  <p> This is the constructor for this class
   * @param iAzimuth  = number of azimuth curves
   * @param iCalipers = number of caliper curves
   * @param degrees   = Angle between the calipers
   * @param stControl = Plot Control Data Structure
   * @param stHeader  = Header Information Data Structure
   * @param stLAS     = LAS File Data Structure
   */

  public lasPlotFrame( int                     iAzimuth,
                       int                     iCalipers,
                       double                  degrees,
	                   iqstratControlStruct    stControl,
                       iqstratHeadersStruct    stHeader,
                       // LAS Data
                       lasFileDataStruct       stLAS )
  {
    try
    {
	  this.iAzimuth  = iAzimuth;
	  this.iCalipers = iCalipers;
	  this.degrees   = degrees;

	  this.stControl = stControl;
	  this.stHeader  = stHeader;
	  this.stLAS     = las.lasFileDataUtility.removeNull(stLAS);

      setTitles( this.stHeader );

      iRows    = count_XYZ( iCalipers, las.lasFileDataUtility.filterBy(this.stLAS, 10.0) );
      data     = convert_XYZ( iAzimuth, iCalipers, degrees,
		                      iRows, las.lasFileDataUtility.filterBy(this.stLAS, 10.0) );
      stLegend = las.lasPlotFilterUtility.getFilter(data[0][2], data[iRows-1][2],
                                             las.lasPlotFilterUtility.TOTAL);
      dStart   = data[0][2];
      dOStart  = data[0][2];
      dEnd     = data[iRows-1][2];
      dOEnd    = data[iRows-1][2];

      stPlot = setPlotStruct( iRows, data, stLegend, sTitle1, sTitle2 );

      jbInit();
      if (panel != null)
      {
		panel.setDepths( dStart,  dEnd);
		panel.setRadius( dRadius );
	  }

      addWindowListener( new lasPlotFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Method jbInit()
   *  <p> This method will create this Frame
   * @throws Exception
   */

  private void jbInit() throws Exception
  {
	int         iWidth       = 0;
	int         iHeight      = 0;

	JPanel      pnlTop       = new JPanel();
	JPanel      pnlMenu      = new JPanel();
	JPanel      pnlBTN       = new JPanel();

    JPanel      pnlDepths    = new JPanel();
    JPanel      pnlStart     = new JPanel();
    JPanel      pnlEnd       = new JPanel();
    JPanel      pnlDepthBTNS = new JPanel();

    JPanel      pnlLimits    = new JPanel();
    JPanel      pnlMax       = new JPanel();
    JPanel      pnlIncr      = new JPanel();

    JPanel      pnlRadius    = new JPanel();

    JPanel      pnlSample    = new JPanel();

    JScrollPane jScrollPane  = new JScrollPane();

    JButton     btnI         = new JButton();
    JButton     btnII        = new JButton();
    JButton     btnIII       = new JButton();
    JButton     btnIV        = new JButton();

    ButtonGroup group        = new ButtonGroup();

    TitledBorder titledBorderStart = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Start [ft]");
    titledBorderStart.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderEnd = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "End [ft]");
    titledBorderEnd.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderRadius = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Borehole");
    titledBorderRadius.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderSample = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Sample every");
    titledBorderSample.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderMax = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "+/- Range");
    titledBorderMax.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    TitledBorder titledBorderIncr = new TitledBorder(
        BorderFactory.createEtchedBorder(Color.white, new Color(165, 163, 151)),
        "Increment");
    titledBorderIncr.setTitleFont(new java.awt.Font("Dialog", 1, 13));

    // Create an Observable

    notifier = new lasPlotFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Panel

    this.getContentPane().setLayout(new BorderLayout());
    this.setTitle( "Borehole Radius Plot" );

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    // Menu Buttons

    pnlMenu.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(29, 29));
    btnClose.setToolTipText("Close Program");
    btnClose.addActionListener(this);

    // -- 1st Separator Button

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    // -- Save Data Buttons

    btnPDF = new JButton(createImageIcon(cmn.cmnStruct.PDF));
    btnPDF.setPreferredSize(new Dimension(29, 29));
    btnPDF.setToolTipText("Create PNG image of Production Plot");
    btnPDF.addActionListener(this);

    // -- 2nd Separator Button

    btnII.setPreferredSize(new Dimension(4, 35));
    btnII.setText("");

    // -- 3rd Separator Button

    btnIII.setPreferredSize(new Dimension(4, 35));
    btnIII.setText("");

    // -- 3rd Separator Button

    btnIV.setPreferredSize(new Dimension(4, 35));
    btnIV.setText("");

    // -- Limits Panel

    pnlLimits.setLayout(new GridLayout());

    // .. +/- Scale Range

    pnlMax.setLayout(new BorderLayout());
    pnlMax.setBorder(titledBorderMax);

    txtMax.setText("" + 10.0);
    txtMax.setColumns(6);
    txtMax.setFont(new java.awt.Font("Dialog", 1, 12));
    txtMax.addFocusListener(new lasPlotFrameFocusAdapter(this));

    // .. Increment

    pnlIncr.setLayout(new BorderLayout());
    pnlIncr.setBorder(titledBorderIncr);

    txtIncr.setText("" + 2.0);
    txtIncr.setColumns(6);
    txtIncr.setFont(new java.awt.Font("Dialog", 1, 12));
    txtIncr.addFocusListener(new lasPlotFrameFocusAdapter(this));

    // -- Depth Range

    pnlDepths.setLayout(new GridLayout());

    // .. Starting Depth

    pnlStart.setLayout(new BorderLayout());
    pnlStart.setBorder(titledBorderStart);

    txtStart.setText("" + dOStart);
    txtStart.setColumns(6);
    txtStart.setFont(new java.awt.Font("Dialog", 1, 12));
    txtStart.addFocusListener(new lasPlotFrameFocusAdapter(this));

    // .. Ending Depth

    pnlEnd.setLayout(new BorderLayout());
    pnlEnd.setBorder(titledBorderEnd);

    txtEnd.setText("" + dOEnd);
    txtEnd.setColumns(6);
    txtEnd.setFont(new java.awt.Font("Dialog", 1, 12));
    txtEnd.addFocusListener(new lasPlotFrameFocusAdapter(this));

    // .. Panel Depth Buttons

    pnlDepthBTNS.setLayout(new GridLayout());

    btnReset.setFont(new java.awt.Font("Dialog", 1, 11));
    btnReset.setPreferredSize(new Dimension(67, 20));
    btnReset.setText("Reset");
    btnReset.addActionListener(this);

    btnModify.setFont(new java.awt.Font("Dialog", 1, 11));
    btnModify.setPreferredSize(new Dimension(71, 20));
    btnModify.setText("Modify");
    btnModify.addActionListener(this);

    // .. Sample Log Data by

    pnlSample.setLayout(new GridLayout());
    pnlSample.setBorder(titledBorderSample);

    rb = new JRadioButton[irb];

    for (int i=0; i<irb; i++)
    {
	  rb[i] = new JRadioButton();
      rb[i].setFont(new java.awt.Font("Dialog", 1, 12));
      rb[i].setText(srb[i]);
      if (i==2) { rb[i].setSelected(true); }
      rb[i].addActionListener(this);

      group.add(rb[i]);
      pnlSample.add(rb[i], null);
    }

    // .. Reference Radius

    pnlRadius.setLayout(new BorderLayout());
    pnlRadius.setBorder(titledBorderRadius);

    txtRadius.setText("" + dRadius);
    txtRadius.setColumns(6);
    txtRadius.setFont(new java.awt.Font("Dialog", 1, 12));
    txtRadius.addFocusListener(new lasPlotFrameFocusAdapter(this));

    // Center Panel

    panel = new lasPlot( 0, stPlot );

    iWidth  =  50 + panel.getPlotWidth();
    iHeight = 150 + panel.getPlotHeight();

    // Attach Widgets

    this.getContentPane().add(pnlTop,      BorderLayout.NORTH);
    pnlTop.add(pnlMenu,                    BorderLayout.NORTH);
    pnlMenu.add(pnlBTN,                    BorderLayout.WEST);
    pnlBTN.add(btnClose,                   null);
    pnlBTN.add(btnI,                       null);
    pnlBTN.add(btnPDF,                     null);

    pnlBTN.add(btnII,                      null);
    pnlBTN.add(pnlSample,                  null);

    pnlBTN.add(btnIII,                     null);
    pnlBTN.add(pnlLimits,                  null);
    pnlLimits.add(pnlMax,                  null);
    pnlMax.add(txtMax,                     BorderLayout.CENTER);

    pnlLimits.add(pnlIncr,                 null);
    pnlIncr.add(txtIncr,                   BorderLayout.CENTER);

    pnlBTN.add(btnIV,                      null);
    pnlBTN.add(pnlDepths,                  null);
    pnlDepths.add(pnlStart,                null);
    pnlStart.add(txtStart,                 BorderLayout.CENTER);

    pnlDepths.add(pnlEnd,                  null);
    pnlEnd.add(txtEnd,                     BorderLayout.CENTER);

    pnlBTN.add(pnlDepthBTNS,               null);
    pnlDepthBTNS.add(btnModify,            null);
    pnlDepthBTNS.add(btnReset,             null);

    pnlMenu.add(pnlRadius,                 BorderLayout.EAST);
    pnlRadius.add(txtRadius,               BorderLayout.CENTER);

    this.getContentPane().add(jScrollPane, BorderLayout.CENTER);
    jScrollPane.getViewport().add(panel,   null);

    // Set Dimension of Frame and Display it.

//    this.setSize(new Dimension(900, 820));
    this.setSize(new Dimension(iWidth, iHeight));
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
//    this.setLocation( (d.width - this.getSize().width)/2,
    this.setLocation( (d.width - this.getSize().width) - 10,
                      (d.height - this.getSize().height)/2);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ================================================================== *
   * ------------------------- CLOSE METHODS -------------------------- *
   * ================================================================== */

  /** METHOD lasPlotFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class lasPlotFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  close();
    }
  }

  /** Method close()
   *  <p> This method will deallocate the global variables to force Java
   *      to free allocated memory.
   */

  public void close()
  {
    // Input Variables

    if (stControl != null)
      stControl.delete();
    stControl = null;

    if (stHeader != null)
      stHeader.delete();
    stHeader = null;

    if (stLAS != null)
      stLAS.delete();
    stLAS  = null;        // LAS File Data Structure

    // Global Variables

    if (stPlot != null)
      stPlot.delete();
    stPlot = null;

    notifier  = null; // Observalbles Object
    panel     = null; // LAS Borehole Plot

    stLegend  = null; // Legend Plot Data Filter Structure

    // -- Save Path & Filename

    sPath     = null;
    sName     = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    if (panel != null)
      panel.close();
    panel     = null;

    // -- raw data

    iRows     = 0;
    data      = null;

    // -- Plot Limits

    sTitle1   = null;
    sTitle2   = null;
    sTitle3   = null;

    sXAxis    = null;
    sYAxis    = null;

    // Global Frame Widgets

    // .. Plot Titles

    txtTitle1 = null;
    txtTitle2 = null;
    txtTitle3 = null;

    // -- Menus Buttons

    btnPDF    = null;
    btnClose  = null;

    // Depth Range

    txtStart  = null;      // Starting Depth
    txtEnd    = null;      // Ending Depth

    btnReset  = null;
    btnModify = null;

    // -- Reference Radius

    txtMax    = null; // Maximum Radius
    txtIncr   = null; // Radius Increment

    // -- Reference Radius

    txtRadius = null;

    // -- Sampling of Log Data

    irb       = 0;
    srb       = null;
    rb        = null;

    dispose();
  }

  /* ===================================================================== *
   * ---------------------------- SET METHODS ---------------------------- *
   * ===================================================================== */

  /** Method setTitles()
   *  <p> This method will set the Initial Plot Titles
   *  @param stHeader = Well Header
   */

  public void setTitles(iqstratHeadersStruct stHeader)
  {
	String sText = "";

	if (stHeader != null)
	{
	  sTitle1  = "Kansas Geological Survey";
	  sTitle2  = new String( stHeader.sName );
	  if (stHeader.sAPI.length() > 1)
	    sTitle2  = new String( sTitle2 + " (" + stHeader.sAPI + ")" );
	}
  }

  /** Method count_XYZ()
   *  <p> This method will count the total number of data points
   *  @param  stLAS = LAS File Data Structure
   *  @return iRows = Total records for plotting
   */

  public static int count_XYZ(lasFileDataStruct stLAS)
  {
	int iRows = 0;

	if (stLAS != null)
	{
	  if (stLAS.iRows > 0)
	  {
		for (int i=0; i<stLAS.iRows; i++)
		{
		  if ((stLAS.iCAL1 > 0) && (stLAS.iAZIC1 > 0))
		  {
			if ((stLAS.dAZIC1[i] != stLAS.dNull) && (stLAS.dCAL1[i] != stLAS.dNull))
			{
			  iRows = iRows + 6;
			}
		  }
		}
	  }
	}

	return (iRows);
  }

  /** Method count_XYZ()
   *  <p> This method will count the total number of data points
   *  @param  iCalipers = total number of calipers
   *  @param  stLAS     = LAS File Data Structure
   *  @return iRows     = Total records for plotting
   */

  public static int count_XYZ(int iCalipers, lasFileDataStruct stLAS)
  {
	int iRows = 0;

	if (stLAS != null)
	{
	  if (stLAS.iRows > 0)
	  {
		for (int i=0; i<stLAS.iRows; i++)
		{
		  if ((stLAS.iCAL1 > 0) && (stLAS.iAZIC1 > 0))
		  {
			if ((stLAS.dAZIC1[i] != stLAS.dNull) && (stLAS.dCAL1[i] != stLAS.dNull))
			{
			  iRows = iRows + iCalipers;
			}
		  }
		}
	  }
	}

	return (iRows);
  }

  /** Method convert_XYZ()
   *  <p> This method will convert the Log data to XYZ Data
   * @param  iRows = Total records for plotting
   * @param  stLAS = LAS File Data Structure
   * @return data  = The XYZ position of Caliper data
   */

  public static double[][] convert_XYZ(int iRows, lasFileDataStruct stLAS)
  {
	double data[][] = null;
	int    iCount   = 0;
	double dAng     = 0.0;
	double dRadian  = 0.0;

	if ((stLAS != null) && (iRows > 0))
	{
	  data = new double[iRows][3];

	  if (stLAS.iRows > 0)
	  {
		for (int i=0; i<stLAS.iRows; i++)
		{
  		  if ((stLAS.dAZIC1[i] != stLAS.dNull) && (stLAS.dCAL1[i] != stLAS.dNull))
		  {
			dAng    = stLAS.dAZIC1[i];
			dRadian = dAng * Math.PI / 180.0;

			data[iCount][2] = stLAS.depths[i];
			data[iCount][0] = stLAS.dCAL1[i] * Math.sin(dRadian);
			data[iCount][1] = stLAS.dCAL1[i] * Math.cos(dRadian);
			iCount++;

			dAng    = stLAS.dAZIC1[i] + 60.0;
			if (dAng > 360.0) { dAng = dAng - 360.0; }
			dRadian = dAng * Math.PI / 180.0;

			data[iCount][2] = stLAS.depths[i];
			data[iCount][0] = stLAS.dCAL2[i] * Math.sin(dRadian);
			data[iCount][1] = stLAS.dCAL2[i] * Math.cos(dRadian);
			iCount++;

			dAng    = stLAS.dAZIC1[i] + 120.0;
			if (dAng > 360.0) { dAng = dAng - 360.0; }
			dRadian = dAng * Math.PI / 180.0;

			data[iCount][2] = stLAS.depths[i];
			data[iCount][0] = stLAS.dCAL3[i] * Math.sin(dRadian);
			data[iCount][1] = stLAS.dCAL3[i] * Math.cos(dRadian);
			iCount++;

			dAng    = stLAS.dAZIC1[i] + 180.0;
			if (dAng > 360.0) { dAng = dAng - 360.0; }
			dRadian = dAng * Math.PI / 180.0;

			data[iCount][2] = stLAS.depths[i];
			data[iCount][0] = stLAS.dCAL4[i] * Math.sin(dRadian);
			data[iCount][1] = stLAS.dCAL4[i] * Math.cos(dRadian);
			iCount++;

			dAng    = stLAS.dAZIC1[i] + 240.0;
			if (dAng > 360.0) { dAng = dAng - 360.0; }
			dRadian = dAng * Math.PI / 180.0;

			data[iCount][2] = stLAS.depths[i];
			data[iCount][0] = stLAS.dCAL5[i] * Math.sin(dRadian);
			data[iCount][1] = stLAS.dCAL5[i] * Math.cos(dRadian);
			iCount++;

			dAng    = stLAS.dAZIC1[i] + 300.0;
			if (dAng > 360.0) { dAng = dAng - 360.0; }
			dRadian = dAng * Math.PI / 180.0;

			data[iCount][2] = stLAS.depths[i];
			data[iCount][0] = stLAS.dCAL6[i] * Math.sin(dRadian);
			data[iCount][1] = stLAS.dCAL6[i] * Math.cos(dRadian);
			iCount++;
		  }
		}
	  }
	}

	return (data);
  }

  /** Method convert_XYZ()
   *  <p> This method will convert the Log data to XYZ Data
   * @param  iAzimuth  = number of azimuth curves
   * @param  iCalipers = number of caliper curves
   * @param  degrees   = Angle between the calipers
   * @param  iRows     = Total records for plotting
   * @param  stLAS     = LAS File Data Structure
   * @return data      = The XYZ position of Caliper data
   */

  public static double[][] convert_XYZ( int iAzimuth, int iCalipers, double degrees,
	                                    int iRows, lasFileDataStruct stLAS)
  {
	double data[][]  = null;
	int    iCount    = 0;
	int    iCurve    = 0;
	double dAng      = 0.0;
	double dRadian   = 0.0;
    double dCaliper  = 0.0;

	if ((stLAS != null) && (iRows > 0))
	{
	  data = new double[iRows][3];

	  if (stLAS.iRows > 0)
	  {
		for (int i=0; i<stLAS.iRows; i++)
		{
  		  if ((stLAS.dAZIC1[i] != stLAS.dNull) && (stLAS.dCAL1[i] != stLAS.dNull))
		  {
		    iCurve = 0;
		    for (int j=0; j<6; j++)
		    {
		  	  if (stLAS.checkData(j))
		  	  {
			    dCaliper = stLAS.getRow(j, i);

  			    dAng    = stLAS.dAZIC1[i] + iCurve * degrees;
			    if (dAng > 360.0) { dAng = dAng - 360.0; }
			    dRadian = dAng * Math.PI / 180.0;

			    data[iCount][2] = stLAS.depths[i];
			    data[iCount][0] = dCaliper * Math.sin(dRadian);
			    data[iCount][1] = dCaliper * Math.cos(dRadian);
			    iCount++;
			    iCurve++;
			  }
		    }
		  }
		}
	  }
	}

	return (data);
  }

  /** Method setPlotStruct()
   * <p> This method will set the Plot Structure
   *  @param  iRows    = Total number of Rows
   *  @param  data     = The XYZ position of Caliper data
   *  @param  stL      = The Plot Filter List Structure
   *  @param  sTitle1  = 1st Title
   *  @param  sTitle2  = 2nd Title
   *  @return stPlotPc = The Plot XY Data Structure
   */

  public static plotXYStruct setPlotStruct(int iRows, double data[][],
                                           lasPlotFilterListStruct stL,
                                           String sTitle1, String sTitle2)
  {
	int i=0;
	double                 dLimit[] = null;
	plotXYStruct           stPlotPc = null;

    String sTitle3  = new String( "" + data[0][2] + "' to " + data[iRows-2][2] + "'" );

    String sXAxis   = "[inches]";
    double dXMin    = -10.0;
    double dXMax    =  10.0;
    double dXIncr   =   2.0;

    String sYAxis   = "[inches]";
    double dYMin    = -10.0;
    double dYMax    =  10.0;
    double dYIncr   =   2.0;

	stPlotPc             = new plotXYStruct();

    stPlotPc.sTitle1     = new String( sTitle1 );
    stPlotPc.sTitle2     = new String( sTitle2 );
    stPlotPc.sTitle3     = new String( sTitle3 );

    stPlotPc.iXLog       = 0;      // X-Axis Semilog Indicator
    stPlotPc.iXCycles    = 1;      // Number of Cycles
    stPlotPc.dXMaximum   = dXMax;  // Maximum X-Axis
    stPlotPc.dXMinimum   = dXMin;  // Minimum X-Axis
    stPlotPc.dXIncrement = dXIncr; // Increment along the X-Axis
    stPlotPc.sXMnemonic  = new String( "" );
    stPlotPc.sXAxis      = new String( sXAxis );

    stPlotPc.iYLog       = 0;      // Y-Axis Semilog Indicator
    stPlotPc.iYCycles    = 1;      // Number of Cycles
    stPlotPc.dYMaximum   = dYMax;  // Maximum Y-Axis
    stPlotPc.dYMinimum   = dYMin;  // Minimum Y-Axis
    stPlotPc.dYIncrement = dYIncr; // Increment along the Y-Axis
    stPlotPc.sYMnemonic  = new String( "" );
    stPlotPc.sYAxis      = new String( sYAxis );

    // Add Legend to Plot Data Structure

    if (stL != null)
    {
	  if (stPlotPc != null)
	  {
  		if (stL.iCount > 0)
  		{
          stPlotPc.bLegend   = true;
          stPlotPc.sLegTitle = new String( "Depth Ranges:" );
		  stPlotPc.stLegend  = new plotSymbolStruct[stL.iCount];
		  stPlotPc.iLegend   = stL.iCount;

		  for (i=0; i<stL.iCount; i++)
		  {
            stPlotPc.stLegend[i] =
                  plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stL.stItem[i].iSymbol,
                    stL.stItem[i].iRed,
                    stL.stItem[i].iGreen,
                    stL.stItem[i].iBlue,
                    stL.stItem[i].symbol );
	      }
		}
	  }
    }

    stPlotPc = addData( iRows, data, stPlotPc, stL );

    return (stPlotPc);
  }

  /** Method addData()
   * <p> This method will add the plot data
   *  @param  iRows    = Total number of Rows
   *  @param  data     = The XYZ position of Caliper data
   *  @param  stPlotPc = The Plot XY Data Structure
   *  @return stPlotPc = The Plot XY Data Structure
   */

  public static plotXYStruct addData( int iRows, double data[][], plotXYStruct stPlotPc,
                                      lasPlotFilterListStruct stL)
  {
    int    i        = 0;
    int    iRed     = 150;
    int    iGreen   = 150;
    int    iBlue    = 150;
	int    iCount   = 0;
    int    iFilter  = -1;

    // Clear the Plot Data Structure

    if (stPlotPc != null)
    {
      stPlotPc.dX = null; // Array of Horizontal Axis Data
      stPlotPc.dY = null; // Array of Vertical Axis Data
      stPlotPc.dZ = null; // Array of Other Data
      stPlotPc.sZ = null; // Array of Description

      if (stPlotPc.stSymbols != null)
      {
	    for(i=0; i<stPlotPc.iRows; i++)
	    {
		  if (stPlotPc.stSymbols[i] != null)
		  {
		    stPlotPc.stSymbols[i].delete();
	    	stPlotPc.stSymbols[i] = null;
	      }
	    }
        stPlotPc.stSymbols = null;
      }
	}

    if (stPlotPc != null)
    {
      stPlotPc.iRows     = iRows;             // Number of Rows
      stPlotPc.depths    = new double[iRows]; // Array of Depths
      stPlotPc.dX        = new double[iRows]; // Array of Horizontal Axis Data
      stPlotPc.dY        = new double[iRows]; // Array of Vertical Axis Data
      stPlotPc.dZ        = new double[iRows]; // Array of Other Data
      stPlotPc.sZ        = new String[iRows]; // Array of Description
      stPlotPc.stSymbols = new plotSymbolStruct[iRows]; // Array of Symbol

      iCount = 0;

      for (i=0; i<iRows; i++)
      {
//		if (isDepth( data ))
        {
	      if (iCount < stPlotPc.iRows)
	      {
			stPlotPc.depths[iCount] = data[i][2];
            stPlotPc.dZ[iCount]     = 0.0;
            stPlotPc.dY[iCount]     = data[i][1];
            stPlotPc.dX[iCount]     = data[i][0];
            stPlotPc.sZ[iCount]     = new String("");

            iFilter = 999;
            iFilter = las.lasPlotFilterUtility.getFilter(data[i][2], data[0][2], data[iRows-1][2],
                                                         las.lasPlotFilterUtility.TOTAL);

            if (iFilter == 999)
            {
              stPlotPc.stSymbols[iCount] =
                plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    plot.plotSymbolStruct.DIAMOND,
                    iRed,
                    iGreen,
                    iBlue,
                    "NONE");
            }
            else
            {
   			  stPlotPc.sZ[iCount] = new String( stL.stItem[iFilter].symbol );
		      stPlotPc.stSymbols[iCount] =
		        plot.plotSymbol.getPlotSymbolStructure(
                    plot.plotSymbolStruct.FILLED,
                    stL.stItem[iFilter].iSymbol,
                    stL.stItem[iFilter].iRed,
                    stL.stItem[iFilter].iGreen,
                    stL.stItem[iFilter].iBlue,
                    stL.stItem[iFilter].symbol);
		    }

            iCount++;
		  }
		}
	  }
	}

	return (stPlotPc);
  }

  /** Method setTitle1()
   * <p> This method will set the first user title
   * @param sText = the first title
   */

  public void setTitle1(String sText)
  {
	panel.setTitle1(sText);
  }

  /** Method setTitle2()
   * <p> This method will set the second user title
   * @param sText = the second title
   */

  public void setTitle2(String sText)
  {
	panel.setTitle2(sText);
  }

  /** Method setTitle3()
   * <p> This method will set the third user title
   * @param sText = the third title
   */

  public void setTitle3(String sText)
  {
	panel.setTitle3(sText);
  }

  /** Method setXAxis()
   * <p> This method will reset the X-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setXAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setXAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setYAxis()
   * <p> This method will reset the Y-Axis Parameters
   * @param sText = The Axis Label
   * @param iGrid = The type of grid to create
   * @param dMax  = The Maximum axis value
   * @param dMin  = The Minimum axis value
   * @param dInc  = The Increment value on the axis - Linear Grid only
   * @param iCycle = The Number of Cycles - Log Grid only
   */

  public void setYAxis(String sText,
                       int    iGrid,
                       double dMax,
                       double dMin,
                       double dInc,
                       int    iCycle)
  {
    panel.setYAxis(sText, iGrid, dMax, dMin, dInc, iCycle);
  }

  /** Method setDefaultDepth()
   * <p> This method will reset the depth to the original LAS Depth
   */

  private void setDefaultDepth()
  {
    txtStart.setText("" + dOStart);
    txtEnd.setText(""   + dOEnd);

    dStart = dOStart;
    dEnd   = dOEnd;

    if (panel != null) { panel.setDepths( dStart,  dEnd); }
  }

  /** Method setDepth()
   * <p> This method will set the Depth Range
   */

  private void setDepth()
  {
    double dTemp = 0.0;

    dStart = cmn.cmnString.stringToDouble(txtStart.getText());
    dEnd   = cmn.cmnString.stringToDouble(txtEnd.getText());

    if (dEnd < dStart)
    {
      dTemp  = dEnd;
      dEnd   = dStart;
      dStart = dTemp;
    }

    // Set the Starting & Ending Depth in the Control Panel

    txtStart.setText("" + dStart);
    txtEnd.setText(""   + dEnd);

    if (panel != null) { panel.setDepths( dStart,  dEnd); }
  }

  /** Method setData()
   *  <p> This method will reload the data with a different sampling rate.
   *  @param iSample = Sampling rate
   */

  private void setData(int iSample)
  {
	switch (iSample)
	{
	  case 0:
	    if (iCalipers == 0)
	    {
          iRows = count_XYZ( stLAS );
          data  = convert_XYZ( iRows, stLAS );
		}
		else
		{
          iRows = count_XYZ( iCalipers, stLAS );
          data  = convert_XYZ( iAzimuth, iCalipers, degrees, iRows, stLAS );
		}
	    break;
	  case 1:
	    if (iCalipers == 0)
	    {
  	      iRows = count_XYZ( las.lasFileDataUtility.filterBy(stLAS, 5.0) );
          data  = convert_XYZ( iRows, las.lasFileDataUtility.filterBy(stLAS, 5.0) );
		}
		else
		{
  	      iRows = count_XYZ( iCalipers, las.lasFileDataUtility.filterBy(stLAS, 5.0) );
          data  = convert_XYZ( iAzimuth, iCalipers, degrees,
			                   iRows, las.lasFileDataUtility.filterBy(stLAS, 5.0) );
		}
	    break;
	  case 2:
	    if (iCalipers == 0)
	    {
          iRows = count_XYZ( las.lasFileDataUtility.filterBy(stLAS, 10.0) );
          data  = convert_XYZ( iRows, las.lasFileDataUtility.filterBy(stLAS, 10.0) );
		}
		else
		{
          iRows = count_XYZ( iCalipers, las.lasFileDataUtility.filterBy(stLAS, 10.0) );
          data  = convert_XYZ( iAzimuth, iCalipers, degrees,
			                   iRows, las.lasFileDataUtility.filterBy(stLAS, 10.0) );
		}
	    break;
	}

    stPlot = setPlotStruct( iRows, data, stLegend, sTitle1, sTitle2 );
	switch (iSample)
	{
	  case 0:
        stPlot.sampling = "All Data";;
	    break;
	  case 1:
	    stPlot.sampling = "Log Sampled Every 5'";
	    break;
	  case 2:
        stPlot.sampling = "Log Sampled Every 10'";
	    break;
	}

    if (panel != null) { panel.setData(stPlot); }
  }

  /* ======================================================================= *
   * ----------------------------- Save Plots ------------------------------ *
   * ======================================================================= */

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

    if (panel != null) image = panel.getImage();

    return (image);
  }

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   */

  private void searchFrame()
  {
	if (sPath.length() == 0)
      sPath = las.io.lasIO.getHomeDir();

    sName   = new String( "plot" );
    pSearch = new guiSearchDirectoryFrame(notifier, sPath, "borehole", ".png");
  }

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Panel
   */

  private void createPDF()
  {
    las.io.lasIO_Files.createImage( sPath, sName, getImage() );
    las.io.lasIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnPDF)    { searchFrame(); }

    // Set Depths

    if (event.getSource() == btnReset)  { setDefaultDepth(); }
    if (event.getSource() == btnModify) { setDepth(); }

	for (int i=0; i<irb; i++)
	{
	  if (event.getSource() == rb[i])
	  {
		setData(i);
	  }
	}

    if (event.getSource() == btnClose)  { close(); }
  }

  /** METHOD focusLost()
   *  <p> This Method will handle the lost focus event for the text fields.
   *  @param e      = Focus Lost Event
   */

  protected void focusLost(FocusEvent e)
  {
	int    iWidget  = -1;
    int    iNumeric = 0;
    String sMessage = new String("");
    String sTemp    = new String("");

    if (e.getSource() == txtStart)
    {
	  iWidget  = 1;
      iNumeric = 1;
      sTemp = txtStart.getText();
      sMessage = new String("Start Depth Value is a Numeric Field");
    }

    if (e.getSource() == txtEnd)
    {
	  iWidget  = 2;
      iNumeric = 1;
      sTemp = txtEnd.getText();
      sMessage = new String("End Depth Value is a Numeric Field");
    }

    if (e.getSource() == txtRadius)
    {
	  iWidget  = 3;
      iNumeric = 1;
      sTemp = txtRadius.getText();
      sMessage = new String("Reference Radius Value is a Numeric Field");
    }

    if (e.getSource() == txtMax)
    {
	  iWidget  = 4;
      iNumeric = 1;
      sTemp = txtMax.getText();
      sMessage = new String("+/- Range Value is a Numeric Field");
    }

    if (e.getSource() == txtIncr)
    {
	  iWidget  = 5;
      iNumeric = 1;
      sTemp = txtIncr.getText();
      sMessage = new String("Increment Value is a Numeric Field");
    }

    if (iNumeric == 1)
    {
      if (!cmn.cmnString.isNumeric(sTemp))
      {
        JOptionPane.showMessageDialog((Component) null,
                                       sMessage,
                                       "ERROR",
                                       JOptionPane.ERROR_MESSAGE);
        switch (iWidget)
        {
		  case 1:
		    txtStart.setText(sTemp);
		    break;
		  case 2:
		    txtEnd.setText(sTemp);
		    break;
		  case 3:
		    txtRadius.setText(sTemp);
		    break;
		  case 4:
		    txtMax.setText(sTemp);
		    break;
		  case 5:
		    txtIncr.setText(sTemp);
		    break;
		}
      }
      else
      {
        if (e.getSource() == txtStart)
          dStart = cmn.cmnString.stringToDouble(txtStart.getText());

        if (e.getSource() == txtEnd)
          dEnd = cmn.cmnString.stringToDouble(txtEnd.getText());

        if (e.getSource() == txtRadius)
        {
          dRadius = cmn.cmnString.stringToDouble(txtRadius.getText());
  		  panel.setRadius( dRadius );
		}

        if ((e.getSource() == txtMax) || (e.getSource() == txtIncr))
        {
  		  panel.setAxes( cmn.cmnString.stringToDouble(txtMax.getText()),
  		                 cmn.cmnString.stringToDouble(txtIncr.getText()) );
  		  panel.setRadius( dRadius );
		}
      }
    }
  }


  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String    sArg = new String( (String) arg);

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS lasPlotFrameFocusAdapter()
  *  <p> This Class will handle Actions when focus is lost on a textfield
  */

class lasPlotFrameFocusAdapter extends java.awt.event.FocusAdapter
{
  lasPlotFrame adaptee;

  lasPlotFrameFocusAdapter(lasPlotFrame adaptee)
  {
    this.adaptee = adaptee;
  }

  public void focusLost(FocusEvent e) { adaptee.focusLost(e); }
}

/** CLASS lasPlotFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class lasPlotFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}


/*
 *  @version 1.1 4/25/2017
 *  @author  John Victorine
 */
